# Copyright 2011 Jason Gerecke
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
# limitations under the License.

import threading
import time


class TaskManager:

	def __init__(self, function=lambda fn: fn(), sleep=0.05):
		self.tasks = []
		self.tasks_lock = threading.Lock()

		self.do = function
		self.sleep = sleep

		self.process_thread = threading.Thread(target=self._run)
		self.process_thread.setDaemon(True)
		self.process_thread.start()

	def add(self, function, arg_list, min_wait = 0, max_wait = float('inf'), priority = 0):
		self.tasks_lock.acquire()
		now = time.time()
		job = (function, arg_list, min_wait + now, max_wait + now)

		if priority == 1:
			self.tasks = [task for task in self.tasks if task[0] != function]
			self.tasks.append(job)
		elif priority == -1:
			if len([task for task in self.tasks if task[0] == function]) == 0:
				self.tasks.append(job)
		elif priority == 0:
			self.tasks.append(job)

		self.tasks_lock.release()

	def process(self):
		self.tasks_lock.acquire()
		tasks = self.tasks[:]
		self.tasks = []
		self.tasks_lock.release()
		for task in tasks:
			now = time.time()
			function, arg_list, after, before = task
			if now < before:
				if now > after:
					function(*arg_list)
				else:
					self.tasks.append(task)

	def _run(self):
		while True:
			if len(self.tasks) > 0:
				self.do(self.process)
			time.sleep(self.sleep)

