#!/bin/bash

# This script will clean up the project directories to get ready for
# distribution, and create a resources folder based on the contents
# of an installation of the original "Desktop Ponies" application.

E_BADARGS=65

if [ $# -ne 2 ]; then
	echo "Usage: `basename $0` {desktop ponies directory} {output directory}"
	exit $E_BADARGS
fi

RES_DIR=$2/resources
SPRITE_DIR=${RES_DIR}/sprite


echo Step 0/5: Verifying required programs are installed
PROGRAMS="cp find gifsicle git mkdir mv pngcrush rm xargs"
for p in $PROGRAMS; do
	which $p > /dev/null
	if [ $? -ne 0 ]; then
		exit 127
	fi
done

echo Step 1/5: Making clean copy in output directory
git checkout-index -a -f --prefix="$2"/
mkdir -p "${SPRITE_DIR}"
cd "$2"

echo Step 2/5: Populating resources directory
cp "$1/Interactions.ini" "${SPRITE_DIR}"
find "$1" -type f -iname "Pony.ini" -printf %h\\0 | xargs -0 -I {} cp -R {} "${SPRITE_DIR}"

echo Step 3/5: Optimizing GIFs
find "${RES_DIR}" -type f -iname "*.gif" -exec gifsicle -O3 --batch "{}" \; -printf "."
echo

echo Step 4/5: Crushing PNGs
find "${RES_DIR}" -type f -iname "*.png" -exec sh -c 'pngcrush -brute -reduce "{}" "{}".new >/dev/null' \; -exec mv "{}".new "{}" \; -printf "."
echo

echo Step 5/5: Firing Orbital Friendship Cannon
find ${RES_DIR} -type f -not -\( -iname "*.gif" -or -iname "*.png" -or -iname "*.ini" -\) -exec rm "{}" \; -printf "."
find . -type d -\( -name "__pycache__" -or -name ".*" -\) -not -name "." -exec rm -rf "{}" \; -printf "."
find . -type f -\( -name "*.pyc" -or -name "*.~" -\) -exec rm "{}" \; -printf "."
echo
